%%%% March 30, 2009
%%%% Bajari, Fox, Kim and Ryan code
%%%% Tested in MATLAB R2008b

%%%% This code estimates, nonparametrically the CDF of the random coefficient 
%%%% distribution. The true distribution is a mixture of two, correlated, bivariate normals.

%%%% The approximating grid of points comes from a Halton sequence and is
%%%% stored in gridR. 
%%%% The estimated weights are saved in the vector theta.

%%%% The estimated values of the CDF of the random coefficients are evaluated 
%%%% over (another, evenly spaced) grid of points. 
%%%% The estimated CDF is saved in the vector estimatedCDFValues.

%%%% Confidence intervals for the estimated weights and the estimated
%%%% CDF are constructed using either Tikhonov regularization through
%%%% generalized cross-validation or subsampling.
%%%% Endpoints for the 95% confidence intervals for theta are stored in
%%%% leftEndPoints and rightEndPoints.
%%%% EndPoints for the 95% confidence intervals for estimatedCDFValues are
%%%% stored in leftEndPointsCDF and rightEndPointsCDF.

%%%% The data generating code is not parallelized. On a multicore machine, the 
%%%% call to lsqlin should be parallelized. 

%%%% Comments should be sent to Jeremy Fox at fox@uchicago.edu
%%%% An Qi contributed to this code.

% Parameters

N = 15000; % number of observations
J = 3;  % number of products (not including outside good)
%K = 2; % number of product characteristics (hard coded in choice of distributions, do not change)
R = 200; % number of grid points in approximation
standardErrorMethod = 0; % select method to construct confidence intervals: 
                         % 0 for Tikhonov regularization 
                         % 1 for subsampling
                         
%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Generate fake data
%%%%%%%%%%%%%%%%%%%%%%%%

% True distribution of the errors

MU = [3 -5;-3 3];
K = size(MU,2);
SIGMA = cat(3,[2 0.3;0.3 .5],[1 -0.2;-0.2 1]);
p = ones(1,2)/2;
obj = gmdistribution(MU,SIGMA,p);

% Plot the true density function (will estimate CDF, not PDF)
ezsurf(@(x,y)pdf(obj,[x y]),[-10 10],[-10 10]);

% Draw values from the true distribution
betas = random(obj,N);

% Generate x's as uniforms
xdata = rand(N,J,K);

% Linear indices: x * beta
linearIndex = diag(betas(:,1)) * xdata(:,:,1);
for k=2:K
    linearIndex = linearIndex + diag(betas(:,k)) * xdata(:,:,k);
end

% Logit choice probabilities

expLinearIndex = exp(linearIndex);
expLinearIndexSum = 1 + sum(expLinearIndex');
choiceProbs = expLinearIndex ./  repmat(expLinearIndexSum,[J,1])' ;

% Add outside share as good J+1

outsideShare = 1 - sum(choiceProbs');
choiceProbsWithOutside = [choiceProbs, outsideShare' ];

% Generate choices

choiceData = zeros(N,1);
for i=1:N
    q = cumsum(choiceProbsWithOutside(i,:))/sum(choiceProbsWithOutside(i,:));
    rs = rand(1);
    choiceData(i) = sum(repmat(rs,1,length(q))>repmat(q,1,1),2)+1;
end


%%%%%%%%%%%%%%%%%%
%%% Estimation
%%%%%%%%%%%%%%%%%%

% Create approximating grid of points for theta using Halton sequence
% Should cover [-10,10] * [-10,10] 
haltonGrid = haltonset(K);
gridR = 10*(net(haltonGrid,R)-0.5);

% Form logit choice probabilities for points on the grid

linearIndexR = zeros(N,J,R);
 for r=1:R
    for k=1:K
    linearIndexR(:,:,r) = linearIndexR(:,:,r) + gridR(r,k) * xdata(:,:,k);
    end
 end
linearIndexR = permute(linearIndexR,[1,3,2]); 

% Logit choice probabilities for all the grid points and observations

expLinearIndexR = exp(linearIndexR);
expLinearIndexSumR = sum(expLinearIndexR,3) + ones(N,R);
choiceProbsR = expLinearIndexR ./repmat(expLinearIndexSumR,[1,1,J]);

% Dependent variables for linear probability model

dependentDataMat = zeros(N,J+1);
for i=1:N
    dependentDataMat(i,choiceData(i)) = 1; 
end

% Rearrange data arrays for regression

dependentDataMatNoOutsideGood = dependentDataMat(:,1:J)';
dependentData = dependentDataMatNoOutsideGood(:);
regData = reshape(permute(choiceProbsR,[3 1 2]),[N*J,R]);

% Minimize the constrained least squares problem

equalityConstraint = ones(R,1);
lowerBounds = zeros(R,1);

options = optimset('MaxIter',10000,'LargeScale','off','Display','off');

[theta,resnorm,residual,exitflag,output,lambda] =  lsqlin(regData,dependentData,[],[],equalityConstraint',1,lowerBounds,[],[],options);

if exitflag ~= 1
    disp('lsqlin does not converge successfully');
end

for r=1:R
    if theta(r)<0
        theta(r)=0;
    end
end

% Grid of points to evaluate CDFs

[b1,b2] = meshgrid(-10:2:10,-10:2:10);
firstIndex = b1(:);
secondIndex = b2(:);
combinedIndex = [firstIndex, secondIndex];

% True CDF values

trueCDFValues = cdf(obj,combinedIndex);

% Estimated CDF values

D = size(combinedIndex,1);
estimatedCDFValues = zeros(D,1);  

for d=1:D
    for r=1:R
        if min(gridR(r,:) <= combinedIndex(d,:))==1
                estimatedCDFValues(d) = estimatedCDFValues(d) + theta(r);
        end 
    end
end

switch standardErrorMethod
    
case {0}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Constructs confidence intervals using 
%%% Tikhonov regularization through generalized cross-validation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Tikhonov Regularization to correct for near-singularity of regData

% Calculation of the Tikhonov parameter using generalized cross-validation
[U,diagonal,junk] = svd(regData,0);
V = @(x) tikhonov(U,diagonal,dependentData,x);
alpha = fminbnd(V,0,10);

% Adjusts variance-covariance matrix for clustering and heteroskedasticity

residuals = dependentData - regData*theta;
X = regData(1:J,:);
e = residuals(1:J);

sumMat = X'*e*e'*X;
for i=2:N
    X = regData((i-1)*J+1:i*J,:);
    e = residuals((i-1)*J+1:i*J);
    sumMat = sumMat + X'*e*e'*X;
end

varianceMat = inv(regData'*regData+alpha*eye(R))*sumMat*inv(regData'*regData+alpha*eye(R));

% Calculates standard errors of theta
stdErrors = sqrt(diag(varianceMat));

% 95% confidence intervals for theta

leftEndPoints = zeros(R,1);
rightEndPoints = zeros(R,1);
for s=1:R
    leftEndPoints(s) = theta(s) - 1.96 * stdErrors(s);
    if (leftEndPoints(s) < 0) 
        leftEndPoints(s) = 0;
    end
    rightEndPoints(s) = theta(s) + 1.96 * stdErrors(s);
    if (rightEndPoints(s) > 1) 
        rightEndPoints(s) = 1;
    end
end;

% Calculates covariance matrix of CDF from covariance matrix of theta
  
CDFVariances = zeros(D,1);
for d=1:D
    u=0;
    v=0;
    for r=1:R
        if min(gridR(r,:) <= combinedIndex(d,:))==1
             u=varianceMat(r,r)+u;
        end
    end
    for r=1:R-1
        for s=r+1:R
             if min(gridR(r,:) <= combinedIndex(d,:))==1
                  if min(gridR(s,:) <= combinedIndex(d,:))==1
                    v=varianceMat(r,s)+v;
                  end
             end
        end
    end
    CDFVariances(d)=u+2*v;
    if estimatedCDFValues(d)<0
        estimatedCDFValues(d)=0;
    end
end

% Calculates standard errors of CDF
CDFStdErrors=sqrt(CDFVariances);

% 95% confidence intervals for CDF

leftEndPointsCDF = zeros(D,1);
rightEndPointsCDF = zeros(D,1);

for d=1:D
    leftEndPointsCDF(d) = estimatedCDFValues(d) - 1.96 * CDFStdErrors(d);
    if (leftEndPointsCDF(d) < 0) 
        leftEndPointsCDF(d) = 0;
    end
    rightEndPointsCDF(d) = estimatedCDFValues(d) + 1.96 * CDFStdErrors(d);
    if (rightEndPointsCDF(d) > 1) 
        rightEndPointsCDF(d) = 1;
    end
end;

case {1}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Constructs confidence intervals using subsampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Subsample Size: 1/4 of total sample
% B will be changed to the nearest multiple of J
B = round(N/4)*J;
if B == 0
    B = J;
end

% Number of Subsamples
M = N-B/J+1;


% Estimated thetas and CDFs for subsamples

thetaMat = zeros(R,M); 
CDFMat = zeros(D,M);

for i=1:M
    
    % Regression data for subsamples
    % The nth subsample contains information for the nth to (n+N/4-1)th
    % observation, with J regression points for each observation
    
    regDatai = zeros(B,R);
    dependentDatai = zeros(B,1);
    for j=1:B
        regDatai(j,:) = regData((i-1)*J+j,:);
        dependentDatai(j) = dependentData((i-1)*J+j);
    end
    
    % Estimated theta for subsamples
    
    [q] =  lsqlin(regDatai,dependentDatai,[],[],equalityConstraint',1,lowerBounds,[],[],options);
    for u=1:R
        if q(u)<0
            q(u)=0;
        end
    end
    thetaMat(:,i) = q;
    
    % Estimated CDF for subsamples
    
    w = zeros(D,1);  
    for d=1:D
        for r=1:R
            if min(gridR(r,:) <= combinedIndex(d,:))==1
                    w(d) = w(d) + q(r);
            end 
        end
    end
    CDFMat(:,i) = w;
    
end

% Constructs confidence intervals using the point estimates for the
% subsamples and the point estimate for the entire sample

% 95% confidence intervals for theta

leftEndPoints=zeros(R,1);
rightEndPoints=zeros(R,1);

for r=1:R
    
    c1 = quantile(sqrt(B)*(thetaMat(r,:)-theta(r)),.975);
    c2 = quantile(sqrt(B)*(thetaMat(r,:)-theta(r)),.025);
    
    leftEndPoints(r)=theta(r)-(1/sqrt(N*J))*c1;
    if (leftEndPoints(r) < 0) 
        leftEndPoints(r) = 0;
    end
    rightEndPoints(r)=theta(r)-(1/sqrt(N*J))*c2;
    if (rightEndPoints(r) > 1) 
        rightEndPoints(r) = 1;
    end
    
end

% 95% confidence intervals for CDF

leftEndPointsCDF=zeros(D,1);
rightEndPointsCDF=zeros(D,1);

for d=1:D
    
    c1 = quantile(sqrt(B)*(CDFMat(d,:)-estimatedCDFValues(d)),.975);
    c2 = quantile(sqrt(B)*(CDFMat(d,:)-estimatedCDFValues(d)),.025);
    
    leftEndPointsCDF(d) = estimatedCDFValues(d)-(1/sqrt(N*J))*c1;
    if (leftEndPointsCDF(d) < 0) 
        leftEndPointsCDF(d) = 0;
    end
    rightEndPointsCDF(d) = estimatedCDFValues(d)-(1/sqrt(N*J))*c2;
    if (rightEndPointsCDF(d) > 1) 
        rightEndPointsCDF(d) = 1;
    end
    
end

end
    



