%%%% March 30, 2009
%%%% Bajari, Fox, Kim and Ryan code
%%%% Tested in MATLAB R2008b

%%%% This code conducts a Monte Carlo study that estimates, nonparametrically 
%%%% the CDF of the random coefficient distribution. The true distribution 
%%%% is a mixture of two, correlated, bivariate normals.

%%%% The approximating grid of points comes from a Halton sequence and is
%%%% stored in gridR. The estimated values of the CDF of the random 
%%%% coefficients are evaluated over (another, evenly spaced) grid of points. 

%%%% Each iteration of the estimator generates estimated weights, 
%%%% estimated values for the CDF, and 95% confidence intervals for the
%%%% estimated weights and the estimated CDF. The confidence intervals are 
%%%% constructed using either Tikhonov regularization through generalized 
%%%% cross-validation or subsampling.

%%%% The estimated weights are saved in thetahats. The estimated values for
%%%% the CDF are saved in estimatedCDFs. Endpoints for the 95% confidence 
%%%% intervals for the thetahats are stored in leftEndPoints and rightEndPoints.
%%%% EndPoints for the 95% confidence intervals for the estimatedCDFs are
%%%% stored in leftEndPointsCDF and rightEndPointsCDF.

%%%% The true CDF is known and is stored in trueCDF. It is not obvious what
%%%% the "true" theta is, since it is part of an approximation to the true 
%%%% CDF. For the true theta, the weights that give the best approximation 
%%%% to the true CDF, given the number of grid points chosen, is used. 
%%%% These weights are generated by running the estimator for a very large 
%%%% sample size (N=10,000) and are stored in trueTheta. The CDF generated
%%%% by trueTheta is stored in trueCDF1.

%%%% The percentage of time in which the "true" theta and the true CDF falls 
%%%% within the bounds of the confidence intervals is calculated. These 
%%%% coverage calculations are stored in coverage for the estimated theta 
%%%% and coverageCDF for the estimated CDF. The coverage calculations for
%%%% the CDF are then repeated using the CDF generated from trueTheta. The
%%%% coverages for trueCDF1 are stored in coverageCDF1.

%%%% The data generating code is not parallelized. On a multicore machine, the 
%%%% call to lsqlin should be parallelized. 

%%%% Comments should be sent to Jeremy Fox at fox@uchicago.edu
%%%% An Qi contributed to this code.

% Parameters

N = 1000; %number of observations
J = 3; %number of products (not including outside good)
%K = 2; % number of product characteristics (hard coded in choice of distributions, do not change)
R = 200; % number of grid points in approximation
M = 100; % number of iterations in the Monte Carlo study
standardErrorMethod = 0; % select method to construct confidence intervals: 
                         % 0 for Tikhonov regularization 
                         % 1 for subsampling

% True distribution of the errors

MU = [3 -5;-3 3];
K = size(MU,2);
SIGMA = cat(3,[2 0.3;0.3 .5],[1 -0.2;-0.2 1]);
p = ones(1,2)/2;
obj = gmdistribution(MU,SIGMA,p);

% Plot the true density function (will estimate CDF, not PDF)
ezsurf(@(x,y)pdf(obj,[x y]),[-10 10],[-10 10]);

% Create approximating grid of points for theta using Halton sequence
% Should cover [-10,10] * [-10,10] 
haltonGrid = haltonset(K);
gridR = 10*(net(haltonGrid,R)-0.5);

% Grid of points to evaluate CDFs

[b1,b2] = meshgrid(-10:2:10,-10:2:10);
firstIndex = b1(:);
secondIndex = b2(:);
combinedIndex = [firstIndex, secondIndex];
D = size(combinedIndex,1);

% True CDF values

trueCDF = cdf(obj,combinedIndex);

% trueTheta contains the weights which give the best approximation to the 
% true CDF, given the number of grid points chosen. trueCDF1
% contains the CDF values derived from trueTheta. 

[trueTheta,trueCDF1,junk1,junk2,junk3,junk4] = estimator(10000,J,K,R,obj,gridR,combinedIndex,D,2);

% Runs Monte Carlo study

[thetahats,estimatedCDFs,leftEndPoints,rightEndPoints,leftEndPointsCDF,rightEndPointsCDF] = estimator(N,J,K,R,obj,gridR,combinedIndex,D,standardErrorMethod);
for m=2:M
    [q,w,e,r,t,y] = estimator(N,J,K,R,obj,gridR,combinedIndex,D,standardErrorMethod);
    thetahats = horzcat(thetahats, q);
    estimatedCDFs = horzcat(estimatedCDFs, w);
    leftEndPoints = horzcat(leftEndPoints, e);
    rightEndPoints = horzcat(rightEndPoints, r);
    leftEndPointsCDF = horzcat(leftEndPointsCDF, t);
    rightEndPointsCDF = horzcat(rightEndPointsCDF, y);
end

% Bias and RMSE calculations for thetas

t = repmat(trueTheta,1,M);
bias = sum(thetahats-t,2)./M;
MSE = sum((thetahats-t).^2,2)./M;
RMSE = sqrt(MSE);

% Coverage calculations for thetas

coverageMat = zeros(R,M);
for m=1:M
    for r=1:R
        if (trueTheta(r) >= leftEndPoints(r,m)) && (trueTheta(r) <= rightEndPoints(r,m))
            coverageMat(r,m)=1;
        end
    end;
end
coverage = sum(coverageMat,2)./M;

% Bias and RMSE calculations for CDFs

t = repmat(trueCDF,1,M);
biasCDF = sum(estimatedCDFs-t,2)./M;
MSECDF = sum((estimatedCDFs-t).^2,2)./M;
RMSECDF = sqrt(MSECDF);

% Coverage calculations for CDFs

coverageMatCDF = zeros(D,M);
for m=1:M
    for d=1:D
        if (trueCDF(d) >= leftEndPointsCDF(d,m)) && (trueCDF(d) <= rightEndPointsCDF(d,m))
            coverageMatCDF(d,m)=1;
        end
    end;
end
coverageCDF = sum(coverageMatCDF,2)./M;

% Repeats CDF calculations using trueCDF1

t1 = repmat(trueCDF1,1,M);
biasCDF1 = sum(estimatedCDFs-t1,2)./M;
MSECDF1 = sum((estimatedCDFs-t1).^2,2)./M;
RMSECDF1 = sqrt(MSECDF1);

coverageMatCDF1 = zeros(D,M);
for m=1:M
    for d=1:D
        if (trueCDF1(d) >= leftEndPointsCDF(d,m)) && (trueCDF1(d) <= rightEndPointsCDF(d,m))
            coverageMatCDF1(d,m)=1;
        end
    end;
end
coverageCDF1 = sum(coverageMatCDF1,2)./M;